"""This is a simple Sudoku solver for finding unique solutions."""

class Board:
    def __init__(self, matrix = None):
        """Create a new Sudoku puzzle from a 9x9 number grid."""
        if matrix is None:
            matrix = [[0] * 9 for i in range(9)]
        self.matrix = [[cell for cell in row] for row in matrix]
        
        
    def consistent(self):
        """Determine whether the puzzle is currently in a consistent state."""
        # check rows
        for y in range(9):
            free = range(10)
            for x in range(9):
                if self.matrix[y][x] and not free[self.matrix[y][x]]:
                    return False
                free[self.matrix[y][x]] = 0

        # check cols
        for x in range(9):
            free = range(10)
            for y in range(9):
                if self.matrix[y][x] and not free[self.matrix[y][x]]:
                    return False
                free[self.matrix[y][x]] = 0

        # check blocks
        for box in range(9):
            free = range(10)
            for cell in range(9):
                value = self.matrix[box/3*3 + cell/3][box%3*3 + cell%3]
                if value and not free[value]:
                    return False
                free[value] = 0
        
        return True
        
    def solved(self):
        """Determine whether the puzzle is completely solved."""
        return all(all(n>0 for n in row) for row in self.matrix)
    
    def missing(self):
        """Count the number of cells that are not yet filled."""
        return sum(sum(int(n is 0) for n in row) for row in self.matrix)
    
    def solvable(self):
        """Determine whether the puzzle is uniquely solvable without affecting the state."""
        if not self.consistent():
            return False
        tester = Board(self.matrix)
        return tester.solve()
        
    def solve(self):
        """Fill in missing cells as far as is possible. Returns True if the puzzle is now complete."""
        if not self.consistent():
            return False
        solving = 1
        while solving:
            solving = self.solve_pass()
        return self.solved()
        
    def __str__(self):
        """Generate a string representation of the puzzle."""
        row_sep = ("+" + ("-" * 3)) * 9 + "+\n"
        row_tpl = "| %s " * 9 + "|\n"
        rows = []
        for row in self.matrix:
            rows.append(row_tpl % tuple(n if n else " " for n in row))
        return row_sep + row_sep.join(rows) + row_sep
        
    def set(self, x, y, value):
        """Set a particular cell to a particular value."""
        if 1 <= value <= 9:
            self.matrix[y][x] = value
        else:
            raise ValueError()
    
    def solve_pass(self):
        """Make one pass at solving. One pass consists of as many elimination passes as possible, and one scanning pass."""
        solving = 1
        solved = 0
        possible = []
        while solving:
            solving, possible = self.solve_eliminate()
            solved += solving
        solved += self.solve_scan(possible)
        return solved
        
            
    def solve_eliminate(self):
        """Fill in cells that have only a single allowed value (one pass). Returns the solved count and the input to solve_scan()."""
        solved = 0
        # Pre-set a 9 * 9 * 0 grid for use as the possible values of each cell.
        possible_values = [[[]] * 9 for i in range(9)]
        for y in range(9):
            for x in range(9):
                # Do not try to solve cells that are already solved.
                if self.matrix[y][x]: continue
                
                # Start out with all possible values
                possible = range(10)
                box, cell = (y/3) * 3 + x/3, y%3 * 3 + x%3
                
                # Scan the row, column and box neighbours of this cell.
                for k in range(9):
                    row_val = self.matrix[y][k] if x != k else 0
                    col_val = self.matrix[k][x] if y != k else 0
                    box_val = self.matrix[box/3*3 + k/3][box%3*3 + k%3] if cell != k else 0
                    possible[row_val] = 0
                    possible[col_val] = 0
                    possible[box_val] = 0
                possible = filter(lambda n:n>0, possible)
                if len(possible) is 1:
                    self.set(x, y, possible[0])
                    solved += 1
                else:
                    possible_values[y][x] = possible
        
        return solved, possible_values
        
    def solve_scan(self, possible_values):
        """Fill in cells that are the only remaining position for a particular value in a particular area. Must receive the input of a solve_eliminate() pass."""
        solved = 0
        
        # Pre-set three 9 * 10 * 9 grids for the possible positions of each
        # number in each row, column or box.
        in_row = [[range(9) for x in range(10)] for y in range(9)]
        in_col = [[range(9) for x in range(10)] for y in range(9)]
        in_box = [[range(9) for x in range(10)] for y in range(9)]
        
        # In each row, find values with only one possible position.
        for y, row in enumerate(in_row):
            for n, number in enumerate(row):
                for x in number:
                    if n not in possible_values[y][x]:
                        number[x] = -1
                number = filter(lambda n: n>=0, number)
                if len(number) is 1:
                    self.set(number[0], y, n)
                    solved += 1

        # In each column, find values with only one possible position.        
        for x, col in enumerate(in_col):
            for n, number in enumerate(col):
                for y in number:
                    if n not in possible_values[y][x]:
                        number[y] = -1
                number = filter(lambda n: n>=0, number)
                if len(number) is 1:
                    self.set(x, number[0], n)
                    solved += 1

        # In each box, find values with only one possible position.
        for i, box in enumerate(in_box):
            for n, number in enumerate(box):
                for cell in number:
                    if n not in possible_values[i/3*3 + cell/3][i%3*3 + cell%3]:
                        number[cell] = -1
                number = filter(lambda n: n>=0, number)
                if len(number) is 1:
                    self.set(i%3*3 + number[0]%3, i/3*3 + number[0]/3, n)
                    solved += 1
        
        # Return the number of solved cells.
        return solved

# Sample:
       
#b = Board([
#[5, 3, 0,  0, 7, 0,  0, 0, 0],
#[6, 0, 0,  1, 9, 5,  0, 0, 0],
#[0, 9, 8,  0, 0, 0,  0, 6, 0],

#[8, 0, 0,  0, 6, 0,  0, 0, 3],
#[4, 0, 0,  8, 0, 3,  0, 0, 1],
#[7, 0, 0,  0, 2, 0,  0, 0, 6],

#[0, 6, 0,  0, 0, 0,  2, 0, 0],
#[0, 0, 0,  4, 1, 9,  0, 0, 5],
#[0, 0, 0,  0, 8, 0,  0, 7, 9],
#])
#print b
#solving = True
#print b.solvable()
#b.solve()
#print b
#print b.consistent()


